#include <amxmodx>
#include <easy_http>
#include <curl>

#define CURL_BUFFER_SIZE 512

new const g_szJsonFileDirectory[] = 		"resource/pluginsList.res"
new const g_szWebsitePluginsListFile[] = 	"http://www.yourwebsite.com/pluginsList.res"
new const g_szWebsitePluginsLink[] = 		"https://www.yourwebsite.com/plugins"

enum pFilesData
{
	pFileArchive[32],
	pFileDir[256],
	pArchiveSize
}

new g_iOldFileSize[1000][pFilesData]
new bool:g_bLogStarted

public plugin_init()
{
	register_plugin("Auto Updater", "1.0", "EFFEX")
	
	register_dictionary("autoupdater.txt")
}

public plugin_cfg()
{
	new EzHttpOptions:opt = ezhttp_create_options()
	ezhttp_ftp_download2(g_szWebsitePluginsListFile, g_szJsonFileDirectory, "downloadFile", EZH_SECURE_EXPLICIT, opt)
}

public downloadFile(EzHttpRequest:request_id)
{
	if(ezhttp_get_error_code(request_id))
	{
		new szErrorMessage[128]
		ezhttp_get_error_message(request_id, szErrorMessage, charsmax(szErrorMessage))
		log_to_file("error.log", "Response download error: %s", szErrorMessage)
	}
	else
	{
		set_task(2.0, "readAndDownload")
	}
}

public readAndDownload()
{
	if(file_exists(g_szJsonFileDirectory))
	{
		new EzJSON:jConfigsFile = ezjson_parse(g_szJsonFileDirectory, true, true)
		if(ezjson_is_object(jConfigsFile))
		{
			new iFileSize = ezjson_object_get_count(jConfigsFile), bool:bIsPlugin
			for(new i, iPluginId, iFileID, EzJSON:iObjectPos, szPluginName[32], szFileName[32], szFileDirectory[128];i < iFileSize;i++)
			{
				iObjectPos = ezjson_object_get_value_at(jConfigsFile, i)

				ezjson_object_get_string(iObjectPos, "filename", szFileName, charsmax(szFileName))
				
				bIsPlugin = bool:(containi(szFileName, ".amxx") != -1)
				if(bIsPlugin)
				{
					ezjson_object_get_string(iObjectPos, "pluginname", szPluginName, charsmax(szPluginName))			
					if((iPluginId = is_plugin_loaded(szPluginName)) == -1)
						continue
				}
				ezjson_object_get_string(iObjectPos, "directory", szFileDirectory, charsmax(szFileDirectory))
				
				if(bIsPlugin)
				{
					new szServerFileName[32], szServerFileVersion[8]
					get_plugin(iPluginId, szServerFileName, charsmax(szServerFileName), .version = szServerFileVersion, .len3 = charsmax(szServerFileVersion))
					if(!equal(szFileName, szServerFileName))
					{
						if(!g_bLogStarted)
						{
							log_to_file("autoupdater.log", "---------- Auto Plugins Updater Start ----------")
							g_bLogStarted = true
						}
						log_to_file("autoupdater.log", "%L", LANG_SERVER, "COULND_UPDATE_PLUGIN", szServerFileName, szFileName)
					}
					else
					{
						new szFileVersion[32]
						ezjson_object_get_string(iObjectPos, "version", szFileVersion, charsmax(szFileVersion))
						if(!equal(szServerFileVersion, szFileVersion))
						{
							if(!g_bLogStarted)
							{
								log_to_file("autoupdater.log", "---------- Auto Plugins Updater Start ----------")
								g_bLogStarted = true
							}
							log_to_file("autoupdater.log", "%L", LANG_SERVER, "COULND_PLUGIN_UPDATED", szFileName, szFileVersion)
							webFileDownload(szFileDirectory, szFileName)
						}
					}
				}
				else 
				{				
					new szDir[256], bool:bLogDownload = ezjson_object_get_bool(iObjectPos, "log")
					formatex(szDir, charsmax(szDir), "%s/%s", szFileDirectory, szFileName)
						
					if(file_exists(szDir))
					{					
						if(bLogDownload)
						{
							g_iOldFileSize[iFileID][pArchiveSize] = file_size(szDir)
						}
						delete_file(szDir)
					}
	
					if(bLogDownload)
					{
						copy(g_iOldFileSize[iFileID][pFileArchive], charsmax(g_iOldFileSize[][pFileArchive]), szFileName)
						copy(g_iOldFileSize[iFileID][pFileDir], charsmax(g_iOldFileSize[][pFileDir]), szDir)
						
						webFileDownload(szFileDirectory, szFileName, iFileID++)
					}
					else webFileDownload(szFileDirectory, szFileName)
				}
			}
			
			if(g_bLogStarted && bIsPlugin)
			{
				log_to_file("autoupdater.log", "---------- Auto Plugins Updater End ----------")
			}
		}
		ezjson_free(jConfigsFile)
	}
}

webFileDownload(szDirectory[], szFile[], iIndex = -1)
{
	new data[2]
	data[0] = fopen(fmt("%s/%s", szDirectory, szFile), "wb")
	data[1] = iIndex
	
	new CURL:curl = curl_easy_init()
	curl_easy_setopt(curl, CURLOPT_BUFFERSIZE, CURL_BUFFER_SIZE)
	curl_easy_setopt(curl, CURLOPT_URL, fmt("%s/%s", g_szWebsitePluginsLink, szFile))
	curl_easy_setopt(curl, CURLOPT_WRITEDATA, data[0])
	curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, "downloadPlugin")
	curl_easy_perform(curl, "checkPluginDownload", data, sizeof(data))
}

public downloadPlugin(data[], size, nmemb, file)
{
	new actual_size = size * nmemb;
	fwrite_blocks(file, data, actual_size, BLOCK_CHAR)
	return actual_size
}

public checkPluginDownload(CURL:curl, CURLcode:code, data[])
{
	fclose(data[0])
	curl_easy_cleanup(curl)
	
	new iIndex = data[1]
	if(iIndex != -1)
	{
		if(g_iOldFileSize[iIndex][pArchiveSize])
		{
			set_task(1.0, "checkNewFileSize", iIndex)
		}
		else
		{
			if(!g_bLogStarted)
			{
				log_to_file("autoupdater.log", "---------- Auto Plugins Updater Start ----------")
				g_bLogStarted = true
			}
				
			log_to_file("autoupdater.log", "%L", LANG_SERVER, "ARCHIVE_DOWNLOADED", g_iOldFileSize[iIndex][pFileArchive])
			log_to_file("autoupdater.log", "---------- Auto Plugins Updater End ----------")
		}
	}
}

public checkNewFileSize(iIndex)
{
	if(g_iOldFileSize[iIndex][pArchiveSize] != file_size(g_iOldFileSize[iIndex][pFileDir]))
	{
		log_to_file("autoupdater.log", "%L", LANG_SERVER, "ARCHIVE_DOWNLOADED", g_iOldFileSize[iIndex][pFileArchive])
		log_to_file("autoupdater.log", "---------- Auto Plugins Updater End ----------")
	}
}
